#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 格式化
# @Time         : 2023/9/4 17:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from langchain.prompts import ChatPromptTemplate, HumanMessagePromptTemplate
# 解析输出并获取结构化的数据
from langchain.output_parsers import StructuredOutputParser, ResponseSchema
from langchain_community.chat_models import ChatOpenAI

response_schemas = [
    ResponseSchema(name="artist", description="The name of the musical artist"),
    ResponseSchema(name="song", description="The name of the song that the artist plays")
]

# 解析器将会把LLM的输出使用我定义的schema进行解析并返回期待的结构数据给我
output_parser = StructuredOutputParser.from_response_schemas(response_schemas)  # output_parser
format_instructions = output_parser.get_format_instructions()

# 这个 Prompt 与之前我们构建 Chat Model 时 Prompt 不同
# 这个 Prompt 是一个 ChatPromptTemplate，它会自动将我们的输出转化为 python 对象
prompt = ChatPromptTemplate(
    messages=[
        HumanMessagePromptTemplate.from_template(
            "Given a command from the user, extract the artist and song names \n \
                                                    {format_instructions}\n{user_prompt}")
    ],
    input_variables=["user_prompt"],
    partial_variables={"format_instructions": format_instructions}
)

artist_query = prompt.format_prompt(user_prompt="I really like So Young by Portugal. The Man")
print(artist_query.messages[0].content)

llm = ChatOpenAI(temperature=0)
artist_output = llm(artist_query.to_messages())
output = output_parser.parse(artist_output.content)
# artist_output = llm.predict(artist_query.to_string())
# output = output_parser.parse(artist_output)

print(output)
print(type(output))
# 这里要注意的是，因为我们





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : glm
# @Time         : 2024/4/22 14:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 后台线程 监控key

"""

from meutils.pipe import *





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : speech
# @Time         : 2023/12/26 13:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.ai_audio.tts import EdgeTTS


class Speech(object):

    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')

    async def acreate(
        self,
        input: str,
        model: Union[str, Literal["tts-1", "tts-1-hd"]] = 'tts',
        voice: Union[str, Literal["alloy", "echo", "fable", "onyx", "nova", "shimmer"]] = '云希',  # 男声 女声
        speed: float = 1,
        **kwargs
    ):
        data = {
            "text": input,
            "voice": voice,
            # "rate": speed - 1
        }
        return EdgeTTS().stream_acreate(**data)


if __name__ == '__main__':
    from meutils.async_utils import async2sync_generator

    text = """
    陕西省，简称“陕”或“秦”，中华人民共和国省级行政区，省会西安，位于中国内陆腹地，黄河中游，东邻山西、河南，西连宁夏、甘肃，南抵四川、重庆、湖北，北接内蒙古，介于东经105°29′—111°15′，北纬31°42′—39°35′之间，总面积205624.3平方千米。 [1] [5]截至2022年11月，陕西省下辖10个地级市（其中省会西安为副省级市）、31个市辖区、7个县级市、69个县。 [121]截至2022年末，陕西省常住人口3956万人。
    """

    # print(cls.find_voices(Locale="zh-CN"))
    for i in async2sync_generator(Speech().acreate(text)):
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : stepchat
# @Time         : 2024/3/26 11:28
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from meutils.notice.feishu import send_message

from chatllm.schemas.kimi_types import KimiData
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse

"https://stepchat.cn/api/proto.chat.v1.ChatMessageService/SendMessageStream"
"https://stepchat.cn/api/proto.chat.v1.ChatService/GenerateChatName"


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')  # refresh_token
        self.access_token = self.get_access_token()

        self.httpx_client = httpx.Client(headers=self.headers, follow_redirects=True)
        self.httpx_aclient = httpx.AsyncClient(headers=self.headers, follow_redirects=True)

    def create(self, request: ChatCompletionRequest):
        request = self.do_request(request)

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}/completion/stream"

        payload = request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        with self.httpx_client.stream("POST", url=url, json=payload) as response:
            for line in response.iter_lines():
                yield from self.do_chunk(line)

    async def acreate(self, request: ChatCompletionRequest):
        request = self.do_request(request)

        # chat_id =request.conversation_id or self.chat_id # 会话失败还得重新建

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}/completion/stream"

        payload = request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        async with self.httpx_aclient.stream("POST", url=url, json=payload) as response:
            async for line in response.aiter_lines():
                for chunk in self.do_chunk(line):
                    yield chunk

    def get_access_token(self):
        url = "https://stepchat.cn/passport/proto.api.passport.v1.PassportService/RegisterDevice"

        response = httpx.post(url=url, json={}, headers=self.headers)

        logger.debug(response.text)
        return response.json().get('accessToken', {}).get('raw')

    @property
    def headers(self):
        headers = {
            'Authorization': f"Bearer {self.api_key}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36',
            "Oasis-Appid": "10200",
            "Oasis-Webid": "084fa0d9cc98dc6b005db9b6e9d24056d0af3532",
            "Oasis-Platform": "web",
            "Oasis-Token": self.api_key,
            "Oasis-DeviceID": "084fa0d9cc98dc6b005db9b6e9d24056d0af3532",


        }
        return headers


if __name__ == '__main__':
    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJhY3RpdmF0ZWQiOnRydWUsImFnZSI6NSwiYmFuZWQiOmZhbHNlLCJleHAiOjE3MTE0MzM1NDAsIm1vZGUiOjIsIm9hc2lzX2lkIjo4MzU0NzEyNzUwMjAxMjQxNiwidmVyc2lvbiI6MX0.HUSmIMGZaQQ7H_wRqiqeFul2rgUrZVtytrKCOanDFKE...eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJhcHBfaWQiOjEwMjAwLCJkZXZpY2VfaWQiOiIwODRmYTBkOWNjOThkYzZiMDA1ZGI5YjZlOWQyNDA1NmQwYWYzNTMyIiwiZXhwIjoxNzEyNzE5NDk3LCJvYXNpc19pZCI6ODM1NDcxMjc1MDIwMTI0MTYsInZlcnNpb24iOjF9.fupY0-Uks4wjFBj3BNtFNff4NUPDKDk7dVWxcRUVBoA"
    access_token = Completions(api_key=token).get_access_token()

    headers = {
        'Authorization': f"Bearer {access_token}",
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',

        "Content-Type": "application/connect+json",

        # "Content-Type": "application/json"

        "Oasis-Appid": "10200",
        "Oasis-Webid": "084fa0d9cc98dc6b005db9b6e9d24056d0af3532",
        "Oasis-Platform": "web",

        "Oasis-Token": access_token,
        "Oasis-DeviceID": "084fa0d9cc98dc6b005db9b6e9d24056d0af3532",

    }
    url = "https://stepchat.cn/api/proto.chat.v1.ChatMessageService/SendMessageStream"
    #
    payload = {"chatId":"83776330600574976","messageInfo":{"text":"11"}}
    #
    response = httpx.post(url, json=payload, headers=headers)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ApiEmbeddings
# @Time         : 2023/8/10 15:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators.retry import retrying

from langchain.embeddings.base import Embeddings


class ApiEmbeddings(BaseModel, Embeddings):
    request_fn: Optional[Callable[[List[str]], List[List[float]]]] = None

    # requests.post('', json={'texts': ['']}).json()['data']

    @retrying
    def _post(self, texts: List[str]) -> List[List[float]]:
        return self.request_fn(texts)

    def embed_documents(self, texts: List[str]) -> List[List[float]]:
        return self._post(texts)

    def embed_query(self, text: str) -> List[float]:
        return self.embed_documents([text])[0]


if __name__ == '__main__':
    ApiEmbeddings().embed_query('xx')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.llmchain.completions import github_copilot
from chatllm.llmchain.completions import moonshot_kimi
from chatllm.llmchain.completions import deepseek

from chatllm.schemas.openai_types import chat_completion_ppu
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
def chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    model = request.model.strip().lower()
    data = request.model_dump()

    # 空服务 按次计费 pay-per-use ppu
    if model.startswith(('pay-per-use', 'ppu')): return chat_completion_ppu

    ############################################################################ 写到外面去
    if request.file_id or model.__contains__('-file-'):  # 'gpt-3.5-turbo-file-d76e7a6b935373a797544628cbc89b76'
        from redis import Redis
        from minio import Minio
        from chatllm.llmchain.applications import ChatFiles

        minio_client = Minio(
            endpoint=os.getenv('MINIO_ENDPOINT'),
            access_key=os.getenv('MINIO_ACCESS_KEY'),
            secret_key=os.getenv('MINIO_SECRET_KEY'),
            secure=False)
        minio_client.presigned_get_object()


        file_map = {} or Redis(**os.getenv("REDIS_CLIENT_PARAMS", {}), decode_responses=True)

        file_id = request.file_id or f"file-{model.split('-file-')[-1]}"
        file_name = file_map.get(file_id)

        logger.debug(file_id)
        logger.debug(file_name)

        file_data = minio_client.get_object('openai', f"{api_key}/{file_name}")
        file_data = io.BytesIO(file_data.read())  # todo: 为啥不支持file_data

        use_ann = request.rag.get('use_ann')
        chunk_size = request.rag.get('chunk_size', 1000)
        response = (
            ChatFiles(
                model=model.split('-file-')[0],
                embedding_model=request.embedding_model,
                openai_api_key=api_key,
                stream=data.get('stream'),
                use_ann=use_ann,
            )
            .load_file(file=file_data, chunk_size=chunk_size)
            .create_sse(query=data.get('messages')[-1].get('content')))  # todo:多轮对话

        return response

    ############################################################################

    if model.startswith(('kimi', 'moonshot')):
        if any(i in model for i in ('web', 'search', 'net')):
            data['use_search'] = True  # 联网模型

        completions = moonshot_kimi.Completions(api_key=api_key)

    elif model.startswith(('deepseek',)):
        completions = deepseek.Completions(api_key=api_key)

    else:
        completions = github_copilot.Completions(api_key=api_key)
        send_message(api_key, title="github_copilot", n=3)

    response: ChatCompletionResponse = completions.create_sse(**data)
    return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : spark
# @Time         : 2023/8/1 12:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

"""
https://github.com/allwefantasy/byzer-llm/blob/df581d8fb69fb3fa3d48782e5bf47797310c37a8/src/byzerllm/saas/sparkdesk/__init__.py#L69  ########
https://github.com/vital121/LLM-Kit/blob/f81a678dbee6db4ed195da17e278083c45c478f4/modules/model/use_api.py#L151
https://github.com/lichuanqi/Python_Learn_Note/blob/c6dc1c47ef9033035802219538bf5ee3c07eef6e/packages/langchain_/llm_.py#L220
"""
import ssl
import hmac
import hashlib
import websocket
import _thread as thread

from urllib.parse import urlparse, urlencode

# ME
from meutils.pipe import *
from chatllm.schemas.openai_api_protocol import *


class SparkBotCompletion(object):

    @classmethod
    def create(
        cls,
        messages: List[Dict[str, Any]],  # [{'role': 'user', 'content': '讲个故事'}]
        **kwargs,
        # {'model': 'gpt-3.5-turbo', 'request_timeout': None, 'max_tokens': None, 'stream': False, 'n': 1, 'temperature': 0.7, 'api_key': 'sk-...', 'api_base': 'https://api.openai-proxy.com/v1', 'organization': ''}
    ):
        appid, api_key, secret_key = kwargs.pop('api_key').split(':')
        url = cls.create_url(api_key, secret_key)

        # WS
        websocket.enableTrace(False)
        ws = websocket.WebSocketApp(
            url,
            on_message=cls.on_message,
            on_open=cls.on_open,
            on_close=cls.on_close,
            on_error=cls.on_error,
        )
        # 参数设置
        ws.messages = messages
        ws.response_queue = Queue()
        kwargs['appid'] = appid
        for k, v in kwargs.items():  # ws.temperature = temperature
            setattr(ws, k, v)

        if kwargs.get('stream'):
            background_tasks.add_task(ws.run_forever, sslopt={"cert_reqs": ssl.CERT_NONE})
            return cls.stream_create(ws.response_queue)

        else:
            ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
            resp = None
            response = {}
            for i, response in enumerate(cls.stream_create(ws.response_queue)):
                if i == 0:
                    resp = response
                    resp['choices'][0]['finish_reason'] = 'stop'
                    resp['choices'][0]['message'] = resp['choices'][0].pop('delta')

                else:
                    resp['choices'][0]['message']['content'] += response['choices'][0]['delta']['content']
            resp['usage'] = response.pop('usage', {})
            return resp

    @staticmethod
    def stream_create(response_queue):  # 取数
        while 1:
            _ = response_queue.get()
            if _ is None: break
            yield _

    @staticmethod
    def on_message(ws, message):
        data = json.loads(message)
        code = data['header']['code']
        if code != 0:
            ws.close()
            raise f'请求错误: {code}, {data}'
        else:
            payload = data["payload"]
            choices = payload["choices"]

            ####################################################################################
            delta = DeltaMessage(**choices["text"][0])

            choice_data = ChatCompletionResponseStreamChoice(
                index=0,
                delta=delta,
                finish_reason=None,
            )

            chunk = ChatCompletionStreamResponse(
                choices=[choice_data],
                model=ws.model,
            ).dict()

            chunk['usage'] = payload.pop('usage', {}).pop('text', {})
            ####################################################################################

            if choices["status"] == 2:
                chunk['choices'][0]['finish_reason'] = 'stop'
                ws.response_queue.put(chunk)  # 存数
                ws.response_queue.put(None)
                ws.close()
            else:
                ws.response_queue.put(chunk)

    @staticmethod
    def on_open(ws):
        thread.start_new_thread(SparkBotCompletion.run, (ws,))

    @staticmethod
    def on_error(ws, error):
        pass

    @staticmethod
    def on_close(ws, a, b):
        pass

    @staticmethod
    def run(ws, *args):
        # 8192
        data = {
            "header": {
                "app_id": ws.appid,
                "uid": "ws.uid",
            },
            "parameter": {
                "chat": {
                    "domain": "generalv2",  # [general,generalv2]
                    "random_threshold": ws.temperature,
                    "max_tokens": ws.max_tokens or 4096,  # 取值为[1,4096]，默认为2048，模型回答的tokens的最大长度
                    "auditing": "default"
                }
            },
            "payload": {
                "message": {
                    "text": ws.messages  # [{'role': 'user', 'content': '讲个故事'}]，所有content的累计tokens需控制8192以内
                }
            }
        }
        data = json.dumps(data)
        ws.send(data)

    @staticmethod
    def create_url(api_key, api_secret):
        from time import mktime
        from datetime import datetime
        from wsgiref.handlers import format_date_time

        # gpt_url = "ws://spark-api.xf-yun.com/v1.1/chat"
        gpt_url = "ws://spark-api.xf-yun.com/v2.1/chat"

        host = urlparse(gpt_url).netloc
        path = urlparse(gpt_url).path

        # 生成RFC1123格式的时间戳
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        # 拼接字符串
        signature_origin = "host: " + host + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "GET " + path + " HTTP/1.1"

        # 进行hmac-sha256进行加密
        signature_sha = hmac.new(api_secret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()

        signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding='utf-8')

        authorization_origin = f'api_key="{api_key}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature_sha_base64}"'

        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')

        # 将请求的鉴权参数组合为字典
        v = {
            "authorization": authorization,
            "date": date,
            "host": host
        }
        # 拼接鉴权参数，生成url
        url = f"{gpt_url}?{urlencode(v)}"
        # 此处打印出建立连接时候的url,参考本demo的时候可取消上方打印的注释，比对相同参数时生成的url与自己代码生成的url是否一致
        return url


if __name__ == '__main__':
    from meutils.pipe import *

    api_key = os.getenv('SPARK_API_KEY')
    # appid, api_key, secret_key = api_key.split(':')
    # print(SparkBotCompletion.create_url(api_key, secret_key))

    kw = {'model': 'gpt-3.5-turbo', 'request_timeout': None, 'max_tokens': None, 'stream': True, 'n': 1,
          'temperature': 0.7, 'api_key': api_key, 'api_base': 'https://api.openai-proxy.com/v1', 'organization': ''}
    g = SparkBotCompletion.create([{'role': 'user', 'content': '1+1'}], **kw)
    rprint(g)
    for i in g:
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : barkup
# @Time         : 2024/1/9 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/openai/openai-python
# todo: 设计更加通用的兜底方案【首先得有靠谱的渠道（多个兜底渠道，作双兜底）】
# 3.5 deepseek
# 遇到内容审核【只要抛错就走备用】
# 完全兼容openai 去掉无关字段
# 重写支持异步

from meutils.pipe import *
from meutils.notice.feishu import send_message

from openai import OpenAI, AsyncOpenAI
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error, completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

send_message = partial(
    send_message,
    title="ChatCompletion主备",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


class MeOpenAI(OpenAI):
    # @default_headers.default_headers.setter
    # def default_headers(self, value):
    #     self._default_headers = value
    @property
    def default_headers(self):
        return {}


@lru_cache()
class Completions(object):
    cache_store = {}

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, **kwargs):
        self.clients = []

        # 需要兜底的模型
        params = dict(
            api_key=api_key or 'sk-...',
            base_url=base_url or 'https://api.githubcopilot.com',
            default_headers={'Editor-Version': 'vscode/1.192.0'},
            max_retries=0,
        )
        # 如果这一步就报错呢
        self.client = OpenAI(**params)  # todo: 异步
        if api_key and api_key.startswith("http"):  # 白嫖
            self.client = MeOpenAI(base_url=api_key)

        self.backup_client = OpenAI(
            api_key=os.getenv("BACKUP_API_KEY"),
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1"),
            max_retries=1,  # 如果还有采用直连
        )

        # 兜底模型映射
        self.backup_llm_map = {
            "backup-llm": "deepseek",  # 兜底

            "backup-gpt": "gpt-3.5-turbo-0125",
            "backup-gpt-4.0": "gpt-4-0125-preview",
        }

    def create(self, request: ChatCompletionRequest, **kwargs):  # 非openai兜底 backup-llm、openai兜底 backup-gpt、特殊标识直接走官转
        data = to_openai_completion_params(request)

        try:
            # 分支：直接走官转：function_call, response_format 等等
            if self.check_model(request):
                # todo: 待确定官转
                data['model'] = "backup-gpt-4" if data['model'].startswith('gpt-4') else "backup-gpt"
                return self.backup_client.chat.completions.create(**data)

            # 分支：非流3.5
            if (
                not request.stream
                and request.model.startswith(('gpt-3.5',))
                and (
                datetime.datetime.now().hour < 6
                or request.messages[-1].get('content', '').startswith('CREATE TABLE')  # todo: 待补全
            )
            ):
                #  默认：如果没有会陷入死循环
                data['model'] = "backup-gpt"

                # 代码类走deepseek-coder
                if request.messages[-1].get('content', '').startswith('CREATE TABLE'):
                    data['model'] = "deepseek-coder"

                return self.backup_client.chat.completions.create(**data)

            # 分支：自动判断
            response = self._create(data)

            data['model'] = "backup-gpt-4" if data['model'].startswith('gpt-4') else "backup-gpt"
            if response:
                if request.stream:  # 流式内容审核:
                    return self.post_process(response, data)
                else:
                    return response
            else:
                return self.backup_client.chat.completions.create(**data)

        except Exception as e:
            e = traceback.format_exc()
            logger.error(e)
            return self._handle_error(data, error=e)

    @timer("Client")
    def _create(self, data):
        try:
            response = self.client.chat.completions.create(**data)  # 流式是懒加载
            return response

        except Exception as e:  # 走兜底
            # e = traceback.format_exc()
            _ = f"CompletionsClient failed: {e} \n\n {data}"  # rate limit exceeded：阶段性跳过这样的请求，防止被封
            send_message(_)
            logging.error(_)

    def post_process(self, response, data):
        """兜底判断
        """
        for chunk in response:
            if chunk.choices and chunk.choices[0].finish_reason == 'content_filter':  # 走兜底
                logger.warning("Content Filter")
                yield from self.backup_client.chat.completions.create(**data)

            yield chunk

    def _handle_error(self, data: Dict[str, Any], error: Optional[str] = None) -> Any:
        send_message(f"兜底出错\n\n{error}\n\n{data}")

        error = "请联系管理员313303303"
        error = "「触发内容审查」，请联系上级管理员"

        if data.get('stream'):
            # Assuming chat_completion_chunk_error is defined elsewhere
            chat_completion_chunk_error.choices[0].delta.content = str(error)
            return (chat_completion_chunk_error,)
        else:
            # Assuming chat_completion_error is defined elsewhere
            chat_completion_error.choices[0].message.content = str(error)
            return chat_completion_error

    @staticmethod
    def check_model(request: ChatCompletionRequest):

        # 直接走官转
        if request.model.startswith(("gpt-4",)):
            if any((request.response_format, request.function_call)):
                return True


if __name__ == '__main__':
    from chatllm.llmchain.completions import github_copilot

    data = {
        'model': 'gpt-4',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': '1+1'}
        ],
        'stream': False
    }

    # data = {'messages': [{
    #                   'content': '## Tools\n\nYou can use these tools below:\n\n### Clock Time\n\nDisplay a clock to show current time\n\nThe APIs you can use:\n\n#### clock-time_getCurrentTime_standalone\n\n获取当前时间\n\n### Realtime Weather\n\nGet realtime weather information\n\nThe APIs you can use:\n\n#### realtime-weather_fetchCurrentWeather\n\n获取当前天气情况\n\n### DALL·E 3\n\nWhenever a description of an image is given, use lobe-image-designer to create the images and then summarize the prompts used to generate the images in plain text. If the user does not ask for a specific number of images, default to creating four captions to send to lobe-image-designer that are written to be as diverse as possible.\n\n  All captions sent to lobe-image-designer must abide by the following policies:\n\n1. If the description is not in English, then translate it.\n2. Do not create more than 4 images, even if the user requests more.\n3. Don\'t create images of politicians or other public figures. Recommend other ideas instead.\n5. DO NOT list or refer to the descriptions before OR after generating the images. They should ONLY ever be written out ONCE, in the prompts field of the request. You do not need to ask for permission to generate, just do it!\n6. Always mention the image type (photo, oil painting, watercolor painting, illustration, cartoon, drawing, vector, render, etc.) at the beginning of the caption. Unless the caption suggests otherwise, make at least 1--2 of the 4 images photos.\n7. Diversify depictions of ALL images with people to include DESCENT and GENDER for EACH person using direct terms. Adjust only human descriptions.\n  - EXPLICITLY specify these attributes, not abstractly reference them.  The attributes should be specified in a minimal way and should directly describe their physical form.\n  - Your choices should be grounded in reality. For example, all of a given OCCUPATION should not be the same gender or race. Additionally, focus on creating diverse, inclusive, and exploratory scenes via the properties you choose during rewrites.  Make choices that may be insightful or unique sometimes.\n  - Use "various" or "diverse" ONLY IF the description refers to groups of more than 3 people. Do not change the number of people requested in the original description.\n  - Don\'t alter memes, fictional character origins, or unseen people. Maintain the original prompt\'s intent and prioritize quality.\n  - Do not create any imagery that would be offensive.\n\n8. Silently modify descriptions that include names or hints or references of specific people or celebrities by carefully selecting a few minimal modifications to substitute references to the people with generic descriptions that don\'t divulge any information about their identities, except for their genders and physiques. Do this EVEN WHEN the instructions ask for the prompt to not be changed. Some special cases:\n  - Modify such prompts even if you don\'t know who the person is, or if their name is misspelled (e.g. "Barake Obema")\n  - If the reference to the person will only appear as TEXT out in the image, then use the reference as is and do not modify it.\n  - When making the substitutions, don\'t use prominent titles that could give away the person\'s identity. E.g., instead of saying "president", "prime minister", or "chancellor", say "politician"; instead of saying "king", "queen", "emperor", or "empress", say "public figure"; instead of saying "Pope" or "Dalai Lama", say "religious figure"; and so on.\n  - If any creative professional or studio is named, substitute the name with a description of their style that does not reference any specific people, or delete the reference if they are unknown. DO NOT refer to the artist or studio\'s style.\n\nThe prompt must intricately describe every part of the image in concrete, objective detail. THINK about what the end goal of the description is, and extrapolate that to what would make satisfying images.\nAll descriptions sent to lobe-image-designer should be a paragraph of text that is extremely descriptive and detailed. Each should be more than 3 sentences long.\n\nThe APIs you can use:\n\n#### lobe-image-designer_text2image____builtin\n\nCreate images from a text-only prompt.',
    #                   'role': 'system'}, {'content': '现在几点\n\n', 'role': 'user'}], 'model': 'gpt-4',
    #  'frequency_penalty': 0.0, 'function_call': None, 'max_tokens': None, 'n': 1, 'presence_penalty': 0.0,
    #  'response_format': None, 'stop': None, 'stream': True, 'temperature': 0.0, 'top_p': 1.0, 'user': None}

    # for i in range(3):
    #     print(Completions().create(ChatCompletionRequest(**data)))
    #     break

    data['stream'] = True

    api_key = github_copilot.Completions.get_access_token('net|ccu_3LuyRKolN7XCsumLWkG8LVYv3rB53eLohMh4')
    print(api_key)
    for i in Completions(api_key=api_key).create(ChatCompletionRequest(**data)):
        print(i)

    # todo: 兜底有问题 。。。。。。。。。。。。。

    # ChatCompletionChunk(id='chatcmpl-8uXP8LrO95xl1UPNKyEdZ1wgOqsQn', choices=[Choice(delta=ChoiceDelta(content=None, function_call=None, role=None, tool_calls=None), finish_reason='stop', index=0, logprobs=None)], created=1708485466, model='gpt-4', object='chat.completion.chunk', system_fingerprint='fp_f084bcfc79')
    # ChatCompletionChunk(id='chatcmpl-8uXPvLSidJJSeM9IBPqjSqvuo4GZL', choices=[Choice(delta=ChoiceDelta(content=None, function_call=None, role=None, tool_calls=None), finish_reason='stop', index=0, logprobs=None, content_filter_offsets={'check_offset': 118, 'start_offset': 118, 'end_offset': 141}, content_filter_results={'hate': {'filtered': False, 'severity': 'safe'}, 'self_harm': {'filtered': False, 'severity': 'safe'}, 'sexual': {'filtered': False, 'severity': 'safe'}, 'violence': {'filtered': False, 'severity': 'safe'}})], created=1708485515, model=None, object=None, system_fingerprint=None)

    import xinference



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat_deeplx
# @Time         : 2024/4/23 18:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.completions.deeplx import Completions

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)
    raw_model = request.model

    response = Completions().create(request)
    if not request.stream:
        response = list(response)[0]
        if request.return_raw_response:  # 返回原生响应
            _ = response.choices[0].message.content
            return json.loads(_)

    return openai_response2sse(response, redirect_model=raw_model)


@router.post("/")  # https://api.deeplx.org/translate
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)
    raw_model = request.model

    response = Completions().create(request)
    if not request.stream:
        response = list(response)[0]
        if request.return_raw_response:  # 返回原生响应
            _ = response.choices[0].message.content
            return json.loads(_)

    return openai_response2sse(response, redirect_model=raw_model)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : Milvus
# @Time         : 2023/7/14 17:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from meutils.pipe import *
from langchain.docstore.document import Document
from langchain_community.vectorstores import Milvus as _Milvus


class Milvus(_Milvus):

    def similarity_search(
        self,
        query: str,
        k: int = 4,
        param: Optional[dict] = None,
        expr: Optional[str] = None,
        timeout: Optional[int] = None,
        threshold: float = 0.5,
        **kwargs: Any,
    ) -> List[Document]:

        if self.col is None:
            logger.debug("No existing collection to search.")
            return []
        docs_scores = self.similarity_search_with_score(
            query=query, k=k, param=param, expr=expr, timeout=timeout, **kwargs
        )

        docs = []
        for doc, score in docs_scores:
            if score > threshold:
                doc.metadata['score'] = round(score, 2)
                docs.append(doc)
        return docs

    def similarity_search_by_batch(self):  # TODO todo
        """
                # todo: batch
        query 前处理
        recall 后处理/精排
        :return:
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : keys4openai
# @Time         : 2024/5/14 16:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 如果有需要增加tokens计算，可放大
2. 平滑+前后置内容
3. 淘汰策略+轮询策略
    前置淘汰
    后置淘汰：根据具体错误
4. 统计信息
5. 日志监控
"""

from meutils.pipe import *
from openai import AsyncClient
import redis

r = redis.Redis(host='localhost', port=6379, db=0)


# np.random.choice()


class PollKeys(object):  # 轮询

    def __init__(self, provider='kimi'):
        self.provider = provider
        self.base_url = f"https://any2chat.chatfire.cn/{self.provider}"

    @classmethod
    def get_next_api_key(cls, provider):
        """轮询"""

        self = cls(provider=provider)
        while True:
            if api_key := r.lpop(self.provider):  # 左边值放在右边
                r.rpush(self.provider, api_key)
                # r.incr(f'api_key_count:{api_key}')  # 增加计数器
                return api_key

    @diskcache(location='check_api_key_cache', ttl=1 * 60, ignore=['self'])
    def check_api_key(self, api_key):
        """定时检测：半小时检测一遍"""
        logger.debug("@@@@@@@@2")
        payload = {
            "token": api_key  # "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9..."
        }
        response = httpx.Client(base_url=self.base_url).post("/token/check", json=payload)
        return response.json()

    def update_api_keys(self, api_keys):
        """定时更新&空闲更新
        1、 通过飞书更新keys
        2、检测key
        3、更新key
        """

        r.delete(self.provider)
        r.rpush(self.provider, *filter(self.check_api_key, api_keys))

    def get_api_keys(self):
        """从飞书"""
        return ['api_key-1', 'api_key-2', 'api_key-3']


if __name__ == '__main__':
    print(str(PollKeys().check_api_key('')))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : llm_test
# @Time         : 2023/12/8 15:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 渠道测试

from meutils.pipe import *

from openai import OpenAI

questions = [
    '树上9只鸟，打掉1只，还剩几只？',
    '鲁迅为什么暴打周树人',
]


def do_chat(
    q='鲁迅为什么暴打周树人',
    api_key: Optional[str] = None,
    base_url: Optional[str] = os.getenv('OPENAI_API_BASE'),
    model='chat2api'
) -> str:
    completions = OpenAI(
        api_key=api_key,
        base_url=base_url,
        max_retries=3
    ).chat.completions

    _ = completions.create(
        messages=[{'role': 'user', 'content': q}],
        model=model,
        temperature=0,
    )

    logger.debug(_)

    return _.choices[0].message.content


if __name__ == '__main__':
    for q in questions:
        print(do_chat(q))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ppu_utils
# @Time         : 2024/1/8 16:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 按次计费
import time

import openai
from openai import OpenAI

from meutils.pipe import *
from chatllm.schemas.openai_types import completion_keys, chat_completion_chunk, chat_completion_chunk_stop
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest

from openai.types.chat import ChatCompletion, ChatCompletionChunk

import tiktoken

token_encoder = tiktoken.get_encoding('cl100k_base')


def to_openai_completion_params(
    request: Union[dict, ChatCompletionRequest],
    redirect_model: Optional[str] = None
) -> dict:
    data = copy.deepcopy(request)
    if isinstance(request, ChatCompletionRequest):
        data = request.model_dump()

    extra_body = {}
    for key in list(data):
        if key not in completion_keys:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段
    data['model'] = redirect_model or data['model']

    return data


# @background_task
def per_create(api_key: Optional[str] = None, base_url: Optional[str] = None, model: str = 'per'):
    openai = OpenAI(
        api_key=api_key,
        base_url=base_url or "https://api.chatllm.vip/v1",
        http_client=httpx.Client(follow_redirects=True)
    )
    return openai.chat.completions.create(model=model, messages=[{"role": "user", "content": "hi"}])


def openai_response2sse(response, redirect_model: Optional[str] = None, n: float = 1.2):
    """

    :param response:
    :param redirect_model: request.model
    :param n:
    :return:
    """
    from sse_starlette import EventSourceResponse

    logger.debug(type(response))

    if isinstance(response, ChatCompletion):
        response.model = redirect_model or response.model

        response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
        completion_tokens = response.usage.completion_tokens or len(response.choices[0].message.content)
        response.usage.completion_tokens = int(completion_tokens * n)
        response.usage.total_tokens = response.usage.prompt_tokens + response.usage.completion_tokens

        return response

    def do_chunk(chunk):
        chunk: ChatCompletionChunk
        chunk.model = redirect_model or chunk.model

        # logger.debug(chunk.choices)

        if chunk.choices and chunk.choices[0].delta.content:  # 跳过首行空值
            if chunk.choices[0].finish_reason == "stop":
                chunk.choices[0].delta.content = ""  # "delta":{} "delta":{"content":""}
                yield chunk.model_dump_json()
                # yield "[DONE]"  # 兼容标准格式
                return
            else:
                yield chunk.model_dump_json()

    if inspect.isasyncgen(response) or isinstance(response, openai.AsyncStream):

        # logger.debug(f"{type(response)}")
        async def gen():
            async for chunk in response:
                # logger.debug(chunk)

                for chunk in do_chunk(chunk):
                    yield chunk
            yield "[DONE]"  # 兼容标准格式

    else:
        def gen():
            for chunk in response:
                for chunk in do_chunk(chunk):
                    yield chunk
            yield "[DONE]"  # 兼容标准格式

    return EventSourceResponse(gen(), ping=10000)  # sse_response


# todo
def do_chunk(chunk, redirect_model):  # 流处理
    if isinstance(chunk, str):
        chat_completion_chunk.choices[0].delta.content = chunk
        yield chat_completion_chunk.model_dump_json()

        # chat_completion_chunk = _chat_completion_chunk.model_copy(deep=True)
        #
        # async for content in self._acreate(request):
        #     chat_completion_chunk.choices[0].delta.content = content
        #     yield chat_completion_chunk
        #
        # # 结束标识
        # chat_completion_chunk.choices[0].delta.content = ""
        # chat_completion_chunk.choices[0].finish_reason = "stop"
        # yield chat_completion_chunk

    # chunk: ChatCompletionChunk
    # chunk.model = redirect_model or chunk.model

    # logger.debug(chunk.choices)

    if chunk.choices and chunk.choices[0].delta.content:  # 跳过首行空值
        if chunk.choices[0].finish_reason == "stop":
            chunk.choices[0].delta.content = ""  # "delta":{} "delta":{"content":""}
            yield chunk.model_dump_json()
            # yield "[DONE]"  # 兼容标准格式
            return
        else:
            yield chunk.model_dump_json()


def openai_response2sse_plus(response, redirect_model: Optional[str] = None, n: float = 1.2):
    """
    兼容迭代器：字符串、字符串生成器、ChatCompletionChunks
    """
    logger.debug(type(response))

    # 非流
    if isinstance(response, ChatCompletion):
        response.model = redirect_model or response.model

        response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
        completion_tokens = response.usage.completion_tokens or len(response.choices[0].message.content)
        response.usage.completion_tokens = int(completion_tokens * n)
        response.usage.total_tokens = response.usage.prompt_tokens + response.usage.completion_tokens

        return response

    # 流

    if inspect.isasyncgen(response) or isinstance(response, openai.AsyncStream):

        # logger.debug(f"{type(response)}")
        async def gen():
            async for chunk in response:
                # logger.debug(chunk)

                for chunk in do_chunk(chunk):
                    yield chunk
            yield "[DONE]"  # 兼容标准格式

    else:
        def gen():
            for chunk in response:
                for chunk in do_chunk(chunk):
                    yield chunk
            yield "[DONE]"  # 兼容标准格式

    from sse_starlette import EventSourceResponse

    return EventSourceResponse(gen(), ping=10000)  # sse_response


if __name__ == '__main__':
    print(per_create())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : cf
# @Time         : 2024/4/30 13:32
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils import chinese_convert
from meutils.apis import deeplx

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk as _chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

from chatllm.llmchain.completions import chatglm_web
from fastapi import UploadFile, File


class Completions(object):
    def __init__(self, api_key: Optional[str] = None):  # @cf/bytedance/stable-diffusion-xl-lightning
        """CLOUDFLARE_ACCOUNT_ID-CLOUDFLARE_API_TOKEN"""

        api_key = api_key or os.getenv("CLOUDFLARE_API_TOKEN")
        cloudflare_account_id, api_key = api_key.split('-', maxsplit=1)

        base_url = f"https://api.cloudflare.com/client/v4/accounts/{cloudflare_account_id}/ai/run/"

        headers = {"Authorization": f"Bearer {api_key}"}

        self.httpx_client = httpx.Client(base_url=base_url, follow_redirects=True, timeout=100, headers=headers)
        self.httpx_aclient = httpx.AsyncClient(base_url=base_url, follow_redirects=True, timeout=100, headers=headers)

    async def acreate(self, request: ChatCompletionRequest):  # todo: minio 兜底
        payload = {
            "prompt": request.last_content
            # "prompt": deeplx.translate(request.last_content).get('data')
        }

        response: httpx.Response = await self.httpx_aclient.post(request.model, json=payload)
        if response.status_code != 200:
            send_message(response.text, title='cf')
            response.raise_for_status()

        # main
        file_object = await chatglm_web.Completions().put_object_for_openai(response.content, "")

        image_url = file_object.filename
        chunk = f"![{request.model}]({image_url})"
        # Path('debug.png').write_bytes(response.content) # debug

        if request.stream:
            chat_completion_chunk = _chat_completion_chunk.model_copy(deep=True)

            # 前置
            chat_completion_chunk.choices[0].delta.content = chunk
            yield chat_completion_chunk

            # 结束标识
            chat_completion_chunk.choices[0].delta.content = ""
            chat_completion_chunk.choices[0].finish_reason = "stop"
            yield chat_completion_chunk

        else:
            chat_completion.usage.completion_tokens = 1
            chat_completion.choices[0].message.content = chunk
            yield chat_completion

    async def acreate_asr(
        self,
        file: Union[UploadFile] = File(...),  # 文件链接
    ):

        payload = {"file": (file.filename, file.file)}  #
        response: httpx.Response = await self.httpx_aclient.post("@cf/openai/whisper", files=payload)

        if response.status_code != 200:
            send_message(response.text, title='cf asr')
            response.raise_for_status()
        _ = response.json().get("result", {"text": ""})
        # logger.debug(_)

        # pip install opencc opencc-python-reimplemented
        _ = json.loads(chinese_convert(json.dumps(_, ensure_ascii=False)))

        # logger.debug(_)

        return _


if __name__ == '__main__':
    request = ChatCompletionRequest(
        model='/@cf/bytedance/stable-diffusion-xl-lightning',
        messages=[{'role': 'user', 'content': '画一条可爱的猫'}],
        stream=False
    )

    response = Completions().acreate(request)
    # print(arun(response))
    #
    # for i in Completions().create(request):
    #     print(i)

    # import tiktoken
    #
    # print(tiktoken.get_encoding('cl100k_base').encode('我是谁'))

    import openai

    # openai.OpenAI.chat.completions.create()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark
# @Time         : 2024/5/15 15:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://www.volcengine.com/product/doubao?utm_source=5&utm_medium=sembaidu&utm_term=sem_baidu_damoxing_doubao_1&utm_campaign=vgbdpcztn797661721486&utm_content=damoxing_doubao
import os

from meutils.pipe import *
import volcenginesdkcore
import volcenginesdkark
from pprint import pprint
from volcenginesdkcore.rest import ApiException

endpoint_map = {
    "doubao-lite-128k": "ep-20240515062839-jsvtz",
    "doubao-lite-32k": "ep-20240515062431-rkhfp",
    "doubao-lite-4k": "ep-20240515062545-nnq8b",

    "doubao-pro-128k": "ep-20240515073409-dlpqp",
    "doubao-pro-32k": "ep-20240515073434-s4skk",
    "doubao-pro-4k": "ep-20240515073524-xj4m2",  # fc

    "doubao-embedding": "ep-20240516005609-9c9pq"
}

if __name__ == '__main__':
    configuration = volcenginesdkcore.Configuration()
    configuration.ak = os.getenv("ARK_ACCESS_KEY")
    configuration.sk = os.getenv("ARK_SECRET_ACCESS_KEY")
    configuration.region = "cn-beijing"
    # set default configuration
    volcenginesdkcore.Configuration.set_default(configuration)

    # use global default configuration
    api_instance = volcenginesdkark.ARKApi()

    # for endpoint in endpoint_map.values():
    get_api_key_request = volcenginesdkark.GetApiKeyRequest(
        duration_seconds=30 * 24 * 3600,
        resource_type="endpoint",
        resource_ids=list(endpoint_map.values()),
        # resource_ids=["ep-20240515062545-nnq8b"]
    )




    try:
        resp = api_instance.get_api_key(get_api_key_request)
        pprint(resp)
    except ApiException as e:
        print("Exception when calling api: %s\n" % e)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2023/4/20 12:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import torch
from transformers import AutoTokenizer, AutoModel

from meutils.pipe import *
from chatllm.utils.gpu_utils import load_chatglm_on_gpus

DEVICE = (
    os.getenv('DEVICE') if 'DEVICE' in os.environ
    else "cuda" if torch.cuda.is_available()
    else "mps" if torch.backends.mps.is_available()
    else "cpu"
)

xgroup = Pipe(lambda ls, step=3, overlap_rate=0: [ls[max(idx - int(step * overlap_rate), 0): idx + step] for idx in
                                                  range(0, len(ls), step)])


def textsplitter(text, chunk_size=512, overlap_rate=0.2, sep=''):  # 简单粗暴
    return text.lower().split() | xjoin(sep) | xgroup(chunk_size, overlap_rate)


def load_llm(model_name_or_path="THUDM/chatglm-6b", device=DEVICE, num_gpus=2, **kwargs):
    model = AutoModel.from_pretrained(model_name_or_path, trust_remote_code=True)
    tokenizer = AutoTokenizer.from_pretrained(model_name_or_path, trust_remote_code=True)

    if torch.cuda.is_available() and device.lower().startswith("cuda"):
        num_gpus = min(num_gpus, torch.cuda.device_count())

        if num_gpus == 1:  # 单卡
            model = model.half().cuda()
            # model.transformer.prefix_encoder.float()
        elif 'chatglm' in model_name_or_path:  # chatglm多卡
            model = load_chatglm_on_gpus(model_name_or_path, num_gpus)
            logger.info('多卡加载模型')

    else:
        model = model.float().to(device)

    return model.eval(), tokenizer


def load_llm4chat(model_name_or_path="THUDM/chatglm-6b", device=DEVICE, num_gpus=2, stream=True, **kwargs):
    model, tokenizer = load_llm(model_name_or_path, device, num_gpus, **kwargs)
    if stream and hasattr(model, 'stream_chat'):
        return partial(model.stream_chat, tokenizer=tokenizer)  # 可以在每一次生成清GPU
    else:
        return partial(model.chat, tokenizer=tokenizer)



if __name__ == '__main__':
    # model, tokenizer = load_llm("/CHAT_MODEL/chatglm", device='cpu')

    # print(completion_keys)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2023/6/30 16:37
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.cache_utils import diskcache, joblib_cache
from openai.resources import Embeddings, Files

Files.create = lru_cache(Files.create)


@lru_cache
def init_cache(verbose=-1):
    CACHE = os.getenv("CHATLLM_CACHE", "~/.cache/chatllm")

    # import openai
    # openai.Embedding.create = diskcache(
    #     openai.Embedding.create,
    #     location=f"{CACHE}/openai.Embedding.create",
    #     verbose=verbose,
    # )

    Embeddings.create = diskcache(
        Embeddings.create,
        location=f"{CACHE}/openai.Embedding.create",
        ignore=['self'],
        verbose=verbose,

    )
    try:

        from sentence_transformers import SentenceTransformer
        SentenceTransformer.encode = diskcache(
            SentenceTransformer.encode,
            location=f"{CACHE}/SentenceTransformer.encode",
            ignore=['self'],
            verbose=verbose,
        )
    except Exception as e:
        logger.warning(e)

    # SentenceTransformer.encode = joblib_cache(
    #     SentenceTransformer.encode,
    #     location=f"{CACHE}__SentenceTransformer",
    #     verbose=verbose,
    # )

    # try:
    #     import dashscope # 返回对象不支持序列化
    #     dashscope.TextEmbedding.call = set_cache(dashscope.TextEmbedding.call, verbose=verbose)
    # except Exception as e:
    #     logger.error(e)

    # 流式会生成不了
    # openai.Completion.create = diskcache(
    #     openai.Completion.create,
    #     location=f"{OPENAI_CACHE}_Completion",
    #     verbose=verbose,
    #     ttl=24 * 3600
    # )
    #
    # openai.ChatCompletion.create = diskcache(
    #     openai.ChatCompletion.create,
    #     location=f"{OPENAI_CACHE}_ChatCompletion",
    #     verbose=verbose,
    #     ttl=24 * 3600
    # )


if __name__ == '__main__':
    from langchain_community.embeddings import OpenAIEmbeddings
    from meutils.pipe import *

    init_cache(1)

    print(OpenAIEmbeddings().embed_query(text='chatllmxxx'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : protocol
# @Time         : 2023/11/29 16:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


class SearchMessage(BaseModel):
    need2search: bool
    search_target: str
    search_language: str


class EventData(BaseModel):
    """
    {"error":{"error_type":"openai.completion.stream","message":"Kimi 现在有点累了，晚一点再来问问我吧！","detail":"runtime error: invalid memory address or nil pointer dereference"},"event":"error"}

    data: {"content":"1+1","event":"req","group_id":"cljdett0tc12f8jcpc7g","id":"cljdett0tc12f8jcpc8g","refs":[]}
    data: {"event":"resp","group_id":"cljdett0tc12f8jcpc7g","id":"cljdett0tc12f8jcpc90"}
    data: {"event":"debug","message":{"need2search":false,"search_target":"","search_language":""}}
    data: {"event":"debug","message":{"need2search":true,"search_target":"南京今天天气","search_language":"zh-cn"}}
    data: {"event":"search_plus","msg":{"type":"start"}}
    data: {"event":"search_plus","msg":{"successNum":1,"title":"南京天气","type":"get_res","url":"http://m.nmc.cn/publish/forecast/AJS/nanjing.html"}}
        {'successNum': 1, 'title': '南京天气预报25天|南京未来25天天气|南京天气预报未来25天...', 'type': 'get_res', 'url': 'http://www.tqw1.com/jsnanjing_25.shtml'}
    data: {"event":"cmpl","text":" "}
    data: {"event":"cmpl","text":"1"}
    data: {"event":"cmpl","text":"å "}
    data: {"event":"done"}
    data: {"event":"cmpl","text":"1"}
    data: {"event":"cmpl","text":"ç­"}
    data: {"event":"cmpl","text":"äº"}
    data: {"event":"cmpl","text":"2"}
    data: {"event":"cmpl","text":"ã"}
    data: {"event":"all_done"}


    """
    # Literal["req", "resp", "debug", "search_plus", "cmpl", "done", "all_done"]
    event: Optional[str] = None
    prompt: str = Field(default='', alias='content')

    id: Optional[str] = None
    group_id: Optional[str] = None

    message: Optional[Any] = None
    msg: dict = {}
    refs: Optional[List[str]] = None

    text: str = ''

    error_type: Optional[str] = None
    # {"error_type":"auth.token.invalid","message":"您的授权已过期，请重新登录"}
    # {"error_type":"chat.forbidden","message":"此会话已被禁用"}

    # json_str = {"messages": [{"role": "system", "content": "你现在扮演的角色是初中老师"},
    #                          {"role": "user", "content": "你现在扮演的角色是？"}], "refs": [], "use_search": False}


if __name__ == '__main__':
    print(EventData(content='###'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : dalle
# @Time         : 2023/11/6 16:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


import streamlit as st

from meutils.pipe import *

# Note: The openai-python library support for Azure OpenAI is in preview.


st.title("图片生成")

with st.form('11'):
    prompt = st.text_area(label='提示词', placeholder='一条鱼')

    if st.form_submit_button('开始生成') and prompt:


        with st.spinner("AI正在生成"):
            import openai

            openai.api_type = "azure"
            openai.api_base = "https://betterme.openai.azure.com/"
            openai.api_version = "2023-06-01-preview"
            openai.api_key = os.getenv("OPENAI_API_KEY", 'a5b4dddb389d4647b72814fe42cae643')

            response = openai.Image.create(
                prompt=prompt,
                size='1024x1024',
                n=1
            )

            image_url = response["data"][0]["url"]

            st.image(image_url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : search
# @Time         : 2024/5/7 09:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://python.langchain.com/docs/integrations/tools/search_tools/

from meutils.pipe import *
from langchain.agents import AgentType, initialize_agent
from langchain_community.chat_models import ChatOpenAI
from langchain.tools import DuckDuckGoSearchRun, SearchAPIRun, GoogleSearchRun


import os

# 替换自己的opensea api
# os.environ["OPENAI_API_KEY"] = 'YOUR_API_KEY'

# 限制了最大请求2048条
llm = ChatOpenAI(temperature=0, max_tokens=2048)

# 这里使用了duckduckgo引擎，使用google自行获取api
web_search = DuckDuckGoSearchRun()

tools = [
    web_search,
    # other tools
]

# 新建代理
agent = initialize_agent(tools, llm, agent=AgentType.ZERO_SHOT_REACT_DESCRIPTION, verbose=True)

# agent.run("周杰伦是谁")


if __name__ == '__main__':
    from langchain_community.utilities import SerpAPIWrapper
    #
    search = SerpAPIWrapper()
    # r = search.run("周杰伦是谁")
    from langchain.agents import AgentType, initialize_agent, load_tools
    from langchain_openai import ChatOpenAI

    llm = ChatOpenAI(temperature=0)
    # tools = load_tools(["searchapi"], llm=llm)
    tools = load_tools(["google-serper"], llm=llm)
    # tools = load_tools(["searx-search"], searx_host="http://localhost:8888", llm=llm)
    agent = initialize_agent(
        tools, llm, agent=AgentType.ZERO_SHOT_REACT_DESCRIPTION, verbose=False
    )
    # agent.run("今天南京天气怎么样")
    for i in agent.stream("今天中国发生的新闻"):
        print(i)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/5/14 17:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


import redis
import time

# 连接到 Redis
r = redis.Redis(host='localhost', port=6379, db=0)

# 初始 API keys 列表
api_keys = ["key1", "key2", "key3", "key4"]


keys = r.lrange('api_keys', 0, -1)
print(keys)

print(r.rpush("keys", *api_keys))
keys = r.lrange('keys', 0, -1)
print(keys)
#
# # 将 API keys 存入 Redis 列表
# r.delete('api_keys')  # 删除旧的 keys 列表
# for key in api_keys:
#     r.rpush('api_keys', key)
#
# def check_api_key(api_key):
#     # 这里应实现实际的 API key 检查逻辑
#     # 返回 True 表示 key 有效，False 表示 key 无效
#     # 示例中，假设 "key3" 是无效的
#     return api_key != "key3"
#
# def poll_api_keys():
#     while True:
#         # 获取所有 API keys
#         keys = r.lrange('api_keys', 0, -1)
#         keys = [key.decode('utf-8') for key in keys]
#         # 检查并淘汰无效的 keys
#         valid_keys = []
#         for key in keys:
#             if check_api_key(key):
#                 valid_keys.append(key)
#
#         print(valid_keys)
#
#
#         # 更新 Redis 列表
#         r.delete('api_keys')
#         for key in valid_keys:
#             r.rpush('api_keys', key)
#
#         # 模拟轮询间隔
#         time.sleep(10)
#
# # 启动轮询
# poll_api_keys()


# from meutils.pipe import *
# import redis
# import schedule
# import time
# import asyncio
#
# # 连接到 Redis
# r = redis.Redis(host='localhost', port=6379, db=0)
#
#
# # 假设我们有一个方法来获取新的 API keys
# def fetch_new_api_keys():
#     # 此处模拟获取新的 API keys 的过程
#     return ["key1", "key2", "key3"]
#
#
# # 定时更新 API keys 的方法
# def update_api_keys():
#     new_keys = fetch_new_api_keys()
#     r.delete('api_keys')  # 删除旧的 keys
#     for key in new_keys:
#         r.rpush('api_keys', key)  # 添加新的 keys
#
#
# # 定时任务，设置每小时更新一次 API keys
# schedule.every().hour.do(update_api_keys)
#
#
# # 检查 API key 是否有效的函数
# def check_api_key(api_key):
#     # 模拟检查过程
#     # 真实场景下可以是一个 HTTP 请求，检查返回的状态码
#     return api_key != "key2"  # 假设 key2 是无效的
#
#
# # 异步轮询 API keys 的方法
# async def poll_api_keys():
#     while True:
#         keys = r.lrange('api_keys', 0, -1)
#         for key in keys:
#             key_str = key.decode('utf-8')
#             if not check_api_key(key_str):
#                 await r.lrem('api_keys', 0, key)  # 移除无效的 key
#                 print(f"Removed invalid key: {key_str}")
#         await asyncio.sleep(60)  # 每分钟轮询一次
#
#
# # 主函数，启动所有任务
# def main():
#     # 启动定时任务的线程
#     def run_schedule():
#         while True:
#             schedule.run_pending()
#             time.sleep(1)
#
#     # 启动异步轮询任务
#     loop = asyncio.get_event_loop()
#     loop.create_task(poll_api_keys())
#
#     # 启动定时任务
#     import threading
#     schedule_thread = threading.Thread(target=run_schedule)
#     schedule_thread.start()
#
#     # 保持主线程运行
#     loop.run_forever()
#
#
# if __name__ == '__main__':
#     main()



#!/usr/bin/env python3
import asyncio
from typing import List, Tuple

from nicegui import Client, ui

messages: List[Tuple[str, str]] = []
contents: List[ui.column] = []


async def update(content: ui.column) -> None:
    content.clear()
    with content:  # use the context of each client to update their ui
        for name, text in messages:
            ui.markdown(f'**{name or "someone"}:** {text}').classes('text-lg m-2')
        await ui.run_javascript(f'window.scrollTo(0, document.body.scrollHeight)', respond=False)


@ui.page('/')
async def main(client: Client):
    async def send() -> None:
        messages.append((name.value, text.value))
        text.value = ''
        await asyncio.gather(*[update(content) for content in contents])  # run updates concurrently

    anchor_style = r'a:link, a:visited {color: inherit !important; text-decoration: none; font-weight: 500}'
    ui.add_head_html(f'<style>{anchor_style}</style>')
    with ui.footer().classes('bg-white'), ui.column().classes('w-full max-w-3xl mx-auto my-6'):
        with ui.row().classes('w-full no-wrap items-center'):
            name = ui.input(placeholder='name').props('rounded outlined autofocus input-class=mx-3')
            text = ui.input(placeholder='message').props('rounded outlined input-class=mx-3') \
                .classes('w-full self-center').on('keydown.enter', send)
        ui.markdown('simple chat app built with [NiceGUI](https://nicegui.io)') \
            .classes('text-xs self-end mr-8 m-[-1em] text-primary')

    await client.connected()  # update(...) uses run_javascript which is only possible after connecting
    contents.append(ui.column().classes('w-full max-w-2xl mx-auto'))  # save ui context for updates
    await update(contents[-1])  # ensure all messages are shown after connecting


ui.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark_openai
# @Time         : 2024/5/15 15:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *

'''
Usage:
Ark v3 sdk
pip install volcengine-python-sdk
'''

from volcenginesdkarkruntime import Ark

# follow this document (https://www.volcengine.com/docs/82379/1263279) to generate API Key
# put API Key into environment variable ARK_API_KEY or specify api_key directly in Ark()
client = Ark()

# if __name__ == "__main__":
#
#     # Non-streaming:
#     print("----- standard request -----")
#     completion = client.chat.completions.create(
#         model="ep-20240515062431-rkhfp",
#         messages=[
#             {
#                 "role": "user",
#                 "content": "Say this is a test",
#             },
#         ],
#     )
#     print(completion.choices[0].message.content)
#
#     # Streaming:
#     print("----- streaming request -----")
#     stream = client.chat.completions.create(
#         model="ep-20240515073434-s4skk",
#         messages=[
#             {
#                 "role": "user",
#                 "content": "1+1",
#             },
#         ],
#         stream=True,
#     )
#     for chunk in stream:
#         if not chunk.choices:
#             continue
#
#         print(chunk.choices[0].delta.content, end="")


if __name__ == '__main__':
    from openai import OpenAI

    client = OpenAI(
        # base_url=os.getenv("ARK_BASE_URL"),
        # api_key=os.getenv("ARK_API_KEY")
    )

    messages = [
        {
            "role": "user",
            "content": "你是誰",
        },
    ]
    r = client.chat.completions.create(
        messages=messages,
        # model="ep-20240515073524-xj4m2",
        model="doubao-pro-4k",
        # model="test",
        # model='hailuo',
        # stream=True,
    )
    print(r)
    for i in r:
        print(i)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ChatAdapter
# @Time         : 2024/5/14 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
# 当然可以，以下是一些其他的名字建议：
#
# 1. **ModelToChat**
# 2. **ChatifyAPI**
# 3. **Conversationalizer**
# 4. **TalkAdaptor**
# 5. **ChatBridge**
# 6. **DialogAdapter**
# 7. **ChatTransform**
# 8. **API2Chat**
# 9. **Conversate**
# 10. **ChatPipeline**
#
# 这些名字都试图捕捉将模型或API转化为聊天模式的核心功能。

from meutils.pipe import *
